
/* estimation des paramtres du modle  hasard constant par intervalle */
libname dur "C:\Users\EDUGUET\FacCours\Cours_2015_2016\M2_M";


options linesize=80;

data duree; set dur.base_handi;
proc contents;
run;

proc freq data=duree; tables duree_hand;
proc freq data=duree; tables duree_hand*d_hand;
run;
proc means;
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=DUREE_HAND; /* variable de dure */
%let d=D_HAND; /* indicatrice de dure complte */
%let outp=_outp;
%let outi=_outi;

/* anne par anne */
%let threshold=1 2 3 4 5 10 15 20 30 40;

proc iml;

start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd;
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

start lv(theta) global(di,yi,range,dc,c,m);
h=exp(theta);
lv1=sum(range[,8]#log(t(h))-t(h)#(range[,10]-range[,6]#(0//c)));
lv2=0;
do j=2 to m;
nj=range[j,6];
rgcj=1:(j-1);
lv2=lv2+nj#(t(h[rgcj])*dc[rgcj]);
end;
lv=lv1-lv2;
return (lv);
finish;


start estim;
print "Estimation of the parameters of the piecewise constant hazard distribution",,;
c=t({&threshold}); call sort(c,1);
nc=nrow(c); m=nc+1;
dc=j(nc,1,0); dc[1]=c[1];
do j=2 to nc; dc[j]=c[j]-c[j-1]; end;
n=nrow(yi);
inter=j(n,1,0);
do i=1 to n;
if yi[i]<=c[1] then inter[i]=1;
               else do; if yi[i]>c[nc] then inter[i]=m;
			                           else do; 
									    do j=2 to nc;
									    if (c[j-1]<yi[i])&(yi[i]<=c[j]) then inter[i]=j;
									    end;
									   end;
			   end;
end;
mi=yi||di||inter;
call sort(mi,1);
yi=mi[,1]; di=mi[,2]; inter=mi[,3];
/* 1st column : intervall number */
/* 2nd column : lower bound */
/* 3rd column : upper bound */
range=j(m,10,0);
range[1,1]=1; range[1,2]=0; range[1,3]=c[1];
do j=2 to nc;
 range[j,1]=j;
 range[j,2]=range[j-1,3]; 
 range[j,3]=c[j];
end;
range[m,1]=m;
range[m,2]=c[nc];
range[m,3]=.; /* infinity */
/* 5th column : last individual index after sorting */
do i=1 to n;
   do j=1 to m;
   if inter[i]=j then range[j,5]=i;
   end;
end;
/* 4th colmun : first individual index after sorting */
range[1,4]=1;
do j=2 to m;
range[j,4]=range[j-1,5]+1;
end;
/* 6th column : number of observations */
range[,6]=range[,5]-range[,4]+j(m,1,1);
/* 7th column : percentage */
range[,7]=range[,6]/range[m,5]#100;
labrange="Intervall"||"gt_Lower"||"le_Upper"||"Block_start"||"Block_end"||"Number_Obs"||
"Percentage"||"Complete_obs"||"Complete_Pct"||"Sum_Durations";
/* 8th column : number of complete observations */
/* 10th column : sum of durations */
do j=1 to m;
rgj=range[j,4]:range[j,5];
dj=di[rgj]; 
range[j,8]=sum(dj);
yj=yi[rgj];
range[j,10]=sum(yj);
end;
/* 9th column : percentage of complete observations */
range[,9]=range[,8]/range[,6]#100;
rangelab=compress(j(m,1,"Int_")+char(range[,1]));
print range[colname=labrange rowname=rangelab];
/* starting value */
h0=j(1,m,nd/sum(yi)); /* exponential case */
theta0=log(h0);
opt=j(11,1,0);
opt[1]=1; opt[2]=5;
call nlpnra(rc,theta,"lv",theta0,opt);
call nlpfdd(lv1,g,he,"lv",theta);
theta=t(theta); g=t(g); elas=abs(theta#g/lv1);
vp=eigval(he);
print "Maximum log-likelihood =" lv1,
      "Gradient =" g  "Elasticities =" elas , "Hessian eigenvalues =" vp;
vtheta=inv(-he);
h=exp(theta);
dg=diag(h);
vh=dg*vtheta*t(dg);
sh=sqrt(vecdiag(vh));
th=abs(h)/sh;
ph=1-probchi(th##2,1);
res_h=h||sh||th||pb;
labc="Hasard"||"Std_Dev"||"Student"||"Pvalue";
labr=compress(j(m,1,"h")+char(range[,1]));
print res_h[colname=labc rowname=labr];
finish estim;

start ih(h,t) global(c,dc,sh); /* hasard and integrated hasard */
m=nrow(h); nc=nrow(c);
do j=1 to m; 
if t<=c[1] then f=(t||h[1]||(h[1]#t)||sh[1]);
          else do; if t>c[nc] then do; 
                              rg=1:nc;
                              f=(t||h[m]||(t(h[rg])*dc[rg]+h[m]#(t-c[nc]))||sh[m]);
							  end;
							  else do;
                              do j=2 to nc;
							  if (c[j-1]<t)&(t<=c[j]) 
                                     then do;
								     rg=1:(j-1);
									 f=(t||h[j]||(t(h[rg])*dc[rg]+h[j]#(t-c[j-1]))||sh[j]);
									 end;
							 end;
							 end;
							 end;
							 end;
return (f);
finish;

start prev;
do time=0.1 to 50 by 0.01;
prev=prev//ih(h,time);
end;
tt=prev[,1];
ht=prev[,2];
iht=prev[,3];
st=exp(-iht);
ct=1-st;
ft=ht#st;
l_ht=ht-1.96#prev[,4];
u_ht=ht+1.96#prev[,4];
labi="&y"||"Hasard"||"Density"||"SDF"||"CDF"||"l_ht"||"u_ht";
resi=tt||ht||ft||st||ct||l_ht||u_ht;
/* median */
ih1=ih(h,range[1,2]); ih2=ih(h,range[1,3]);
s=(exp(-ih1[1,3])||exp(-ih2[1,3]));
do j=2 to nc;
ih1=ih(h,range[j,2]); ih2=ih(h,range[j,3]);
s=s//(exp(-ih1[1,3])||exp(-ih2[1,3]));
end;
if min(s)>0.5 then do;
              rg=1:nc;
              md=c[nc]+(log(2)-t(h[rg])*dc[rg])/h[m];
			  end;
			  else do;
			  do j=1 to nc;
			  if (s[j,1]>0.5)&(0.5>=s[j,2]) then do ;
                                            rg=1:(j-1);
                                            md=c[j-1]+(log(2)-t(h[rg])*dc[rg])/h[j];
								            end;
			  end;
			  end;
print "Median duration =" md;
/* estimation of the mean */
mt=0; c=0//c;
do j=1 to m-1; /* attention : dcalage d'indices */
mt=mt+(exp(-h[j]#c[j])-exp(-h[j]#c[j+1]))/h[j];
end;
mt=mt+exp(-h[m]#c[m])/h[m];
print "Mean duration =" mt;
finish prev;

start sortie;
create &outi from resi[colname=labi]; append from resi;
create &outp from res_h[colname=labc]; append from res_h;
create _np from labr[colname="Statistic"]; append from labr;
finish sortie;

run lecture;
run estim;
run prev;
run sortie;
quit;
run;
data &outp; merge _np &outp;
run;


/* graphique 1 */
goptions htext=1.5;
symbol1 c=black i=stepjL line=1;
symbol2 c=black i=stepjL line=33;
symbol3 c=black i=stepjL line=33;
proc gplot data=_outi;
plot (hasard l_ht u_ht)*&y/overlay;
run;

proc gplot data=_outi;
plot density*&y/overlay;
run;


data tab1; set _outi;
keep duree_hand sdf Fonction;
Fonction="S(t)";
rename sdf=f;
run;

data tab2; set _outi;
keep duree_hand cdf Fonction;
Fonction="F(t)";
rename cdf=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=stepjl v=none line=3;
symbol2 c=black i=stepjl v=none line=1;
proc gplot data=tab;
plot f*duree_hand=fonction/grid;
run;

/* graphique 2 */

data tab1; set _outi;
keep duree_hand hasard Fonction;
Fonction="h(t)";
rename hasard=f;
run;

data tab2; set _outi;
keep duree_hand density Fonction;
Fonction="f(t)";
rename density=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=stepjl v=none line=3;
symbol2 c=black i=stepjl v=none line=1;
proc gplot data=tab;
plot f*duree_hand=fonction/grid;
run;

